/*****************************************************************************/
/* sample1.c                                                                 */
/*                                                                           */
/* This program provides examples of how to use the features provided by     */
/* version 1.0 and 2.0 of the ARM API.                                       */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>
#include "arm.h"

int32 appl_id = -1;  	/* Define an indentifer for the application id     */

int32 simple_tran_id = -1; /* Define a unique identifier for each  */
int32 long_tran_id_1 = -1; /* TRANSACTION                          */
int32 long_tran_id_2 = -1;
int32 sub_tran_id_1 = -1;
int32 sub_tran_id_2 = -1;



/*****************************************************************************/
/* init                                                                      */
/*****************************************************************************/

void init(void)
{
   appl_id=arm_init("ARM sample program",   /* application name */
                    "*",                   /* use default user */
                    0,0,0);      

   simple_tran_id = arm_getid(appl_id,      
                              "Simple_transaction_1", /* transaction name */
                              "First Transaction in Sample program",
                              0,0,0); 
   if (simple_tran_id < 0) 
        printf("Simple_transaction_1 is not registered.\n");

   long_tran_id_1 = arm_getid(appl_id,      
                              "Long_transaction_1", /* transaction name */
                              "A long transaction using arm_update",
                              0,0,0);
   if (long_tran_id_1 < 0) 
        printf("Long_transaction_1 is not registered.\n");

   long_tran_id_2 = arm_getid(appl_id,      
                              "Long_transaction_2", /* transaction name */
                              "A long transaction using sub transactions",
                              0,0,0);
   if (long_tran_id_2 < 0) 
        printf("Long_transaction_2 is not registered.\n");

   sub_tran_id_1 = arm_getid(appl_id,      
                              "Sub_tran1_of_long_tran_2", /* transaction name */
                              "Subtransaction 1 of Long_trans2",
                              0,0,0); 
   if (sub_tran_id_1 < 0) 
        printf("Sub_tran_of_long_tran_2 is not registered.\n");

   sub_tran_id_2 = arm_getid(appl_id,      
                              "Sub_tran2_of_long_tran_2", /* transaction name */
                              "Subtransaction 2 of Long_trans2",
                              0,0,0); 
   if (sub_tran_id_2 < 0) 
        printf("Sub_tran_of_long_tran_2 is not registered.\n");


} /* init */



/*****************************************************************************/
/* simple_trans1                                                             */
/*****************************************************************************/

void simple_trans1(void)
{
   int32  tran_handle;

   tran_handle = arm_start(simple_tran_id, /* transaction id from arm_getid */
                           0,0,0);   

   /*********************************************/
   /* Perform actual transaction processing here*/
   /*********************************************/

   arm_stop(tran_handle,          /* transaction handle from arm_start */
            ARM_GOOD,             /* successful completion define = 0 */
            0,0,0);               
   return;
}  /* simple_trans1 */



/*****************************************************************************/
/* long_trans_using_update                                                   */
/*                                                                           */
/* arm_update can show the progress of an iterative process                  */
/*****************************************************************************/

void long_trans_using_update(void)
{

#define MAX_COUNT     1000000
#define UPDATE_COUNT  100000  /* call update every 100,000 iterations */

   int32  tran_handle;
   int  i;

   tran_handle = arm_start(long_tran_id_1, /* transaction id from arm_getid */
                           0,0,0);  

   for (i=1;i<=MAX_COUNT;i++)
   {
       /* your processing goes here */

       if (i%UPDATE_COUNT == 0) 
         arm_update(tran_handle,  /* update based on UPDATE_COUNT */
                    0,0,0);
   }

   arm_stop(tran_handle,          /* transaction handle from arm_start */
            ARM_GOOD,             /* successful completion define = 0 */
            0,0,0);               
   return;

}  /* long_trans_using_update */



/*****************************************************************************/
/* long_trans_using_sub_trans                                                */
/*                                                                           */
/* Sub-transactions can show the progress of the steps of a long transaction.*/
/*****************************************************************************/

void long_trans_using_sub_trans(void)
{
   int32  tran_handle;
   int32  sub_tran_handle1;
   int32  sub_tran_handle2;

   /* record the overall transaction processing (optional) */

   tran_handle = arm_start(long_tran_id_2, /* transaction id from arm_getid */
                           0,0,0);

   /* start recording the first step of the long transaction */
   sub_tran_handle1 = arm_start(sub_tran_id_1,
                                0,0,0);
   
   /**************************************/
   /* Process step 1 on this transaction */
   /**************************************/

 
   /* record the completion of the first step */
   arm_stop(sub_tran_handle1,      /* transaction handle from arm_start */
            ARM_GOOD,              /* successful completion define = 0 */
            0,0,0);              
   

   /* start recording the second step of the long transaction */
   sub_tran_handle2 = arm_start(sub_tran_id_2,
                                0,0,0);
   
   /**************************************/
   /* Process step 2 on this transaction */
   /**************************************/


   /* record the completion of the second step */
   arm_stop(sub_tran_handle2,      /* transaction handle from arm_start */
            ARM_GOOD,              /* successful completion define = 0 */
            0,0,0);             
   

   /* record the completion of the overall transaction */
   arm_stop(tran_handle,           /* transaction handle from arm_start */
            ARM_GOOD,              /* successful completion define = 0 */
            0,0,0);            
   return;
}  /* long_trans_using_sub_trans */



/*****************************************************************************/
/* main                                                                      */
/*****************************************************************************/

main()
{

   int continue_processing = 1;

   init();

   while (continue_processing) 
   {
       simple_trans1();
       long_trans_using_update();
       long_trans_using_sub_trans();

       continue_processing = 0;
   }

   arm_end(appl_id,               /* application id from arm_init */
           0,0,0);            
   
   return(0);

}
