
        IDENTIFICATION DIVISION.
        program-id. TT-COBOL-TEST.

        ENVIRONMENT DIVISION.

        DATA DIVISION.

        WORKING-STORAGE SECTION.

        01 appname       pic x(20).
        01 appl-user-id  pic x(20).
        01 appl-id       pic s9(9) comp  VALUE IS -1.
        01 trans-name    pic x(20).
        01 trans-detail  pic x(30).
        01 sim-tran-id   pic s9(9) comp  VALUE IS -1.
        01 lon-tran-id1  pic s9(9) comp  VALUE IS -1.
        01 lon-tran-id2  pic s9(9) comp  VALUE IS -1.
        01 sub-tran-id1  pic s9(9) comp  VALUE IS -1.
        01 sub-tran-id2  pic s9(9) comp  VALUE IS -1.
        01 tag           pic s9(9) comp  VALUE IS -1.
        01 tran-handle   pic s9(9) comp  VALUE IS -1.
        01 sub-handle1   pic s9(9) comp  VALUE IS -1.
        01 sub-handle2   pic s9(9) comp  VALUE IS -1.
        01 dsize         pic s9(9) comp  VALUE IS 0.
        01 flags         pic 9(9)  comp  VALUE IS 0.
        01 trans-data      USAGE POINTER VALUE NULL.
        01 slpval        pic 9(9)  comp  VALUE IS 5.
        01 loop-count    pic 9(9)  comp  VALUE IS 0.
        01 register-flag pic 9(9)  comp  VALUE IS 0.
        01 init-flag     pic 9(9)  comp  VALUE IS 0.
        01 trans-error   pic 9(9)  comp  VALUE IS 0.
        01 ARM-GOOD      pic 9(9)  comp  VALUE IS 0.
        
        PROCEDURE DIVISION.

        MAIN-PROCEDURE.

      *  COMMENT
      *  Load the arm dll if running from WINDOWS.
            call "libarm.dll".
      
      *  Start main procedure
            perform initialize-transaction.
            if init-flag = 1 then
              perform register-transaction
              if register-flag = 1 then
                perform simple-transaction 
                
                move 0 to trans-error
                perform long-update-transaction 

                move 0 to trans-error
                perform long-sub-transaction 
              end-if.
              perform end-transaction.
                
            stop run.

        end-program.
        
        
        initialize-transaction section.           
        
      *  Initialize the transation
           move "ARM sample program" to appname.
           move "*" to appl-user-id.
           call "arm_init" using
                by reference        appname,
                by reference        appl-user-id,
                by value            flags,
                by reference        trans-data,
                by value            dsize
                   returning        appl-id.

           if appl-id < 0 then
              display "Initialization Error:"
              display appname
           else 
              move 1 to init-flag.

      
        end-transaction section.          
      
      *  End of transation

           call "arm_end" using 
                by value            appl-id,
                by value            flags,
                by reference        trans-data,
                by value            dsize
                   returning        tag.

           if tag < 0 then
              display "Error in ending transaction"
           end-if. 


        register-transaction section.

      *  Register all the transation to be used in this program

           move "Simple_transaction_1" to trans-name
           call "arm_getid" using
                by value            appl-id,
                by reference        trans-name,
                by value            flags, 
                by reference        trans-data,
                by value            dsize
                   returning        sim-tran-id.
    
           if sim-tran-id < 0 then
              display "Registration Error:"
              display trans-name

      *  If you do not want to run this application without transaction
      *  information being logged, you need to resolve the problem of
      *  transaction not being registered.

           else
              move 1 to register-flag.

           move "Long_transaction_1" to trans-name
           call "arm_getid" using
                by value            appl-id,
                by reference        trans-name,
                by value            flags, 
                by reference        trans-data,
                by value            dsize
                   returning        lon-tran-id1.
    
           if lon-tran-id1 < 0 then
              display "Registration Error:"
              display trans-name
           else
              move 1 to register-flag.

           move "Long_transaction_2" to trans-name
           call "arm_getid" using
                by value            appl-id,
                by reference        trans-name,
                by value            flags, 
                by reference        trans-data,
                by value            dsize
                   returning        lon-tran-id2.
    
           if lon-tran-id2 < 0 then
              display "Registration Error:"
              display trans-name
           else
              move 1 to register-flag.

           move "Sub_tran1_of_long_tran_2" to trans-name
           call "arm_getid" using
                by value            appl-id,
                by reference        trans-name,
                by value            flags, 
                by reference        trans-data,
                by value            dsize
                   returning        sub-tran-id1.
    
           if sub-tran-id1 < 0 then
              display "Registration Error:"
              display trans-name
           else
              move 1 to register-flag.

           move "Sub_tran2_of_long_tran_2" to trans-name
           call "arm_getid" using
                by value            appl-id,
                by reference        trans-name,
                by value            flags, 
                by reference        trans-data,
                by value            dsize
                   returning        sub-tran-id2.
    
           if sub-tran-id2 < 0 then
              display "Registration Error:"
              display trans-name
           else
              move 1 to register-flag.


       simple-transaction section.
      
      *  Section processing the simple transation

           call "arm_start" using
                by value            sim-tran-id,
                by value            flags,
                by reference        trans-data,
                by value            dsize
                   returning        tag.

           if tag < 0 then
              display "Error returned from arm_start"
              move 1 to trans-error
           end-if.

      *
      *  Process simple transaction code.
      *  Any processing that needs to be done for this
      *  transaction should come here. 
      *

           if trans-error = 0 then
               call "arm_stop" using
                    by value            tag,
                    by value            ARM-GOOD,
                    by value            flags,
                    by reference        trans-data,
                    by value            dsize
                       returning        tag.

           if tag < 0 then
              display "Error returned from arm_stop"
           end-if.


       long-update-transaction section.
      
      *  Section processing the long transaction with
      *  updates of the transaction values inbetween
      *
      *  Record the start of the long transaction

          if trans-error = 0 then 
              call "arm_start" using
                   by value            lon-tran-id1,
                   by value            flags,
                   by reference        trans-data,
                   by value            dsize
                      returning        tran-handle.

           if tran-handle < 0 then
              display "Error returned from arm_start"
              move 1 to trans-error 
           else
              move 0 to loop-count
              perform update-transaction 
                 until loop-count is greater than 10.

           if trans-error = 0 then
               call "arm_stop" using
                    by value            tran-handle,
                    by value            ARM-GOOD,
                    by value            flags,
                    by reference        trans-data,
                    by value            dsize
                       returning        tag.

           if tag < 0 then
              display "Error returned from arm_stop"
           end-if.

       long-sub-transaction section.
      
      *  Section processing the long transaction with
      *  sub transactions inbetween
      *
      *  Record the overall transaction processing
           call "arm_start" using
                by value            lon-tran-id2,
                by value            flags,
                by reference        trans-data,
                by value            dsize
                   returning        tran-handle.

           if tran-handle < 0 then
              display "Error from arm_start of long transaction"
              move 1 to trans-error
           end-if.

      *  Start recording the first step of the long transaction
           if trans-error = 0 then
             call "arm_start" using
                by value            sub-tran-id1,
                by value            flags,
                by reference        trans-data,
                by value            dsize
                   returning        sub-handle1.

           if sub-handle1 < 0 then
              display "Error from arm_start of sub transaction 1"
              move 1 to trans-error
           end-if.

      *
      *  Process step 1 on this transaction
      *  Any processing that needs to be done for this step of
      *  of the transaction should come here. 
      *

      *  Record the completion of the first step
           if trans-error = 0 then
               call "arm_stop" using
                    by value            sub-handle1,
                    by value            ARM-GOOD,
                    by value            flags,
                    by reference        trans-data,
                    by value            dsize
                       returning        tag.

           if tag < 0 then
              display "Error from arm_stop of sub transaction 1"
              move 1 to trans-error
           end-if.

      *  Start recording the second step of the long transaction
           if trans-error = 0 then
             call "arm_start" using
                by value            sub-tran-id2,
                by value            flags,
                by reference        trans-data,
                by value            dsize
                   returning        sub-handle2.

           if sub-handle2 < 0 then
              display "Error from arm_start of sub transaction 2"
              move 1 to trans-error
           end-if.

      *
      *  Process step 2 on this transaction
      *  Any processing that needs to be done for this step of
      *  of the transaction should come here. 
      *

      *  Record the completion of the second step
           if trans-error = 0 then
               call "arm_stop" using
                    by value            sub-handle2,
                    by value            ARM-GOOD,
                    by value            flags,
                    by reference        trans-data,
                    by value            dsize
                       returning        tag.

           if tag < 0 then
              display "Error from arm_stop of sub transaction 1"
              move 1 to trans-error
           end-if.

      *  Record the completion of the overall transaction
           if trans-error = 0 then
               call "arm_stop" using
                    by value            tran-handle,
                    by value            ARM-GOOD,
                    by value            flags,
                    by reference        trans-data,
                    by value            dsize
                       returning        tag.

           if tag < 0 then
              display "Error from arm_stop of overall transaction"
              move 1 to trans-error
           end-if.


       update-transaction section.
      *
      *  Update an existing transaction
      *
           call "arm_update" using
               by value            tran-handle,
               by value            flags,
               by reference        trans-data,
               by value            dsize
                  returning        tag.
           
           if tag < 0 then
              display "Error returned from arm_update"
           end-if.

          add 1 to loop-count.
               

